from node import Node


class LinkedList:
    def __init__(self):
        self.head = None
        self.tail = None

    # Θ(1)
    def push_front(self, data):
        new_node = Node(data)
        new_node.set_next(self.head)
        if self.is_empty():
            self.tail = new_node
        self.head = new_node

    # Θ(1)
    def push_back(self, data):
        new_node = Node(data)
        if self.is_empty():
            self.head = new_node
        else:
            self.tail.set_next(new_node)
        self.tail = new_node

    # Θ(1)
    def pop_front(self):
        obj = self.head.get_data()
        if self.head is self.tail:
            self.head = None
            self.tail = None
        else:
            self.head = self.head.get_next()
        return obj

    # Θ(n)
    def size(self):
        n = 0
        current = self.head
        while current is not None:
            current = current.get_next()
            n += 1
        return n

    # Θ(1)
    def is_empty(self):
        return self.head is None


if __name__ == "__main__":
    lst = LinkedList()  # -> lst: []
    for i in range(10):
        lst.push_back(i)  # -> lst: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
    for _ in range(5):
        print(lst.pop_front())  # -> 0, 1, 2, 3, 4; lst: [5, 6, 7, 8, 9]
    print('---')
    for i in range(-1, -4, -1):
        lst.push_front(i)  # lst: [-3, -2, -1, 5, 6, 7, 8, 9]
    print('---')
    while not lst.is_empty():
        print(lst.pop_front())  # -3, -2, -1, 5, 6, 7, 8, 9; lst: []
